﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Configuration;
using System.Web.Configuration;
using System.Web.UI.HtmlControls;
using System.Data;
using System.Data.SqlClient;

namespace COM.StartCoding
{
    public partial class _Default : System.Web.UI.Page
    {
        private const int I_COLSPAN = 5;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                txtConnString.Text = getConnString();
            }
        }

        private string getConnString()
        {
            Configuration config = WebConfigurationManager.OpenWebConfiguration(HttpContext.Current.Request.ApplicationPath);
            foreach (ConnectionStringSettings connection in config.ConnectionStrings.ConnectionStrings)
            {
                if (connection.Name.Equals("AdventureWorks"))
                {
                    return connection.ConnectionString;
                }
            }

            return "";
        }

        protected void btnGetData_Click(object sender, EventArgs e)
        {
            bindListView(getData(txtConnString.Text, Convert.ToInt32(txtColumnCount.Text)));
        }

        public DataTable getData(string connstr, int iVendorCnt)
        {
            DataTable dt = new DataTable();
            DataTable dtEmp = execSProc(connstr, iVendorCnt);

            ViewState["cachedTable"] = dtEmp;

            dt = dtEmp.Clone();// clone schema only
            DataRow[] drResults = dtEmp.Select("GENERATION = 0");

            foreach (DataRow dr in drResults)
            {
                object[] row = dr.ItemArray;
                dt.Rows.Add(row);
            }

            return dt;
        }

        private DataTable execSProc(string connstr, int iVendorCnt)
        {
            using (SqlConnection conn = new SqlConnection(connstr))
            {
                SqlCommand cmd = new SqlCommand("GetTabularCustomerVSVendor", conn);
                cmd.Parameters.Add("@VendorCount", SqlDbType.Int).Value = iVendorCnt;
                cmd.CommandType = CommandType.StoredProcedure;
                SqlDataAdapter adp = new SqlDataAdapter(cmd);
                DataTable dt = new DataTable();
                adp.Fill(dt);
                return dt;
            }
        }

        #region listView
        public void bindListView(DataTable dtEmpID)
        {
            DataTable dt = new DataTable();
            dt = ((DataTable)ViewState["cachedTable"]).Clone();

            if (((DataTable)ViewState["cachedTable"]).Rows.Count > 0)
            {
                DataRow[] drResults = ((DataTable)ViewState["cachedTable"]).Select("GENERATION = 0");

                foreach (DataRow dr in drResults)
                {
                    object[] row = dr.ItemArray;
                    dt.Rows.Add(row);
                }
            }

            lsvHighLvlFormAccess.DataSource = dt;
            lsvHighLvlFormAccess.DataBind();
        }

        protected void lsvHighLvlFormAccess_DataBound(object sender, EventArgs e)
        {
            PlaceHolder phDynamicHdr = (PlaceHolder)lsvHighLvlFormAccess.FindControl("phDynamicHdr");
            // check phDynamicHdr.Controls.Count; could be called twice when postback
            if (phDynamicHdr != null /*&& phDynamicHdr.Controls.Count == 0*/)
            {
                Literal ltrl = new Literal();

                DataTable dt = ((DataTable)ViewState["cachedTable"]);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataColumn dc in dt.Rows[0].Table.Columns)
                    {
                        if (!(dc.ColumnName.Equals("GENERATION") ||
                                dc.ColumnName.Equals("hierarchy") ||
                                dc.ColumnName.Equals("rowNo") ||
                                dc.ColumnName.Equals("EmployeeID")))
                        {
                            if (dc.ColumnName.Equals("LoginID"))
                            {
                                ltrl.Text += "<th colspan=" + (I_COLSPAN + 1) + ">" + dc.ColumnName + "</th>";
                            }
                            else
                                ltrl.Text += "<th>" + dc.ColumnName + "</th>";
                        }
                    }
                }

                if (phDynamicHdr.Controls.Count > 0)
                {
                    // if current dynamic columns is different with exisiting dynamic columns 
                    if (!((Literal)phDynamicHdr.Controls[0]).Text.Equals(ltrl.Text))
                    {
                        // remove the whole previous dynamic columns
                        phDynamicHdr.Controls.Remove(phDynamicHdr.Controls[0]);
                        // replace with new dynamic columns
                        phDynamicHdr.Controls.Add(ltrl);
                    }
                }
                else
                    phDynamicHdr.Controls.Add(ltrl);
            }

            HtmlTableCell td = (HtmlTableCell)lsvHighLvlFormAccess.FindControl("imgCollapseExpand");
            if (td != null)
            {
                DataTable dt = ((DataTable)ViewState["cachedTable"]);
                if (dt != null && dt.Rows.Count > 0)
                {
                    td.ColSpan = dt.Rows[0].Table.Columns.Count + I_COLSPAN - 1;//I_COLSPAN - 1 : to put the 'minus.png'
                }
            }
        }

        protected int getRowCount(object sHierarchy, object sGeneration)
        {
            DataRow[] drResults = new DataRow[] { };
            if (!string.IsNullOrEmpty(sHierarchy.ToString()))
            {
                drResults = ((DataTable)ViewState["cachedTable"]).Select("hierarchy like '" + sHierarchy.ToString() + " ->%'");
            }
            return drResults.Count();
        }

        protected void lsvHighLvlFormAccess_ItemDataBound(object sender, ListViewItemEventArgs e)
        {
            HtmlTableRow row = (HtmlTableRow)e.Item.FindControl("row");
            // http://msdn.microsoft.com/en-us/library/system.web.ui.webcontrols.listviewdataitem.dataitem.aspx
            // http://forums.asp.net/t/1334142.aspx/1
            ListViewDataItem item = (ListViewDataItem)e.Item;
            System.Data.DataRowView drv = (System.Data.DataRowView)item.DataItem;

            dynamicPopulateRow(row, drv, 0);

            var lst1stLevel = (ListView)e.Item.FindControl("lst1stLevel");
            populateLV(lst1stLevel, 1, (string)drv["hierarchy"], Convert.ToInt32(drv["rowNo"]));
        }

        private void dynamicPopulateRow(HtmlTableRow row, System.Data.DataRowView drv, int iGeneration)
        {
            if (row != null)
            {
                // http://www.pcreview.co.uk/forums/do-enumerate-all-columns-dataviewrow-t1244448.html
                foreach (DataColumn dc in drv.Row.Table.Columns)
                {
                    string sEmployeeID = drv["LoginID"].ToString();

                    if (dc.ColumnName.Equals("LoginID"))
                    {
                        // http://msdn.microsoft.com/en-US/library/e5daxzcy(v=vs.80).aspx
                        // Define a new HtmlTableCell control.
                        HtmlTableCell cell = new HtmlTableCell("td");

                        // Create the text for the cell.
                        cell.Controls.Add(new LiteralControl(Convert.ToString(drv[dc.ColumnName])));
                        cell.ColSpan = dc.ColumnName.Equals("LoginID") ? I_COLSPAN - iGeneration : 1;

                        // Add the cell to the HtmlTableRow Cells collection. 
                        row.Cells.Add(cell);
                    }
                    else if (!(dc.ColumnName.Equals("GENERATION") ||
                                dc.ColumnName.Equals("hierarchy") ||
                                dc.ColumnName.Equals("rowNo") ||
                                dc.ColumnName.Equals("EmployeeID")))
                    {
                        // http://msdn.microsoft.com/en-US/library/e5daxzcy(v=vs.80).aspx
                        // Define a new HtmlTableCell control.
                        HtmlTableCell cell = new HtmlTableCell("td");

                        bool bIsNull = drv[dc.ColumnName] is System.DBNull;

                        Literal ltrl = new Literal();
                        ltrl.Text += "<input type=\"checkbox\" name=\"" + dc.ColumnName + "\"" +
                                        (bIsNull ? "" : " value=" + drv[dc.ColumnName].ToString()) +
                                        " id=\"" + sEmployeeID + "~" + dc.ColumnName.Replace(" ", "_") + "\"" +//will be retrieved later
                                        " onclick=\"didModify(this)\" " +
                                        (bIsNull ? " disabled" : "") +
                                        (!bIsNull && ((int)drv[dc.ColumnName]) > 0 ? " checked>" : ">");

                        cell.Controls.Add(ltrl);
                        // Add the cell to the HtmlTableRow Cells collection. 
                        row.Cells.Add(cell);
                    }
                    else
                    {
                        //other rows
                    }
                }
            }
        }

        private void populateLV(ListView lv, int iNextGeneration, string sHierarchy, int iCurrRowNo)
        {
            if (lv != null)
            {
                DataTable dt = new DataTable();
                dt = ((DataTable)ViewState["cachedTable"]).Clone();// clone schema only

                //http://stackoverflow.com/questions/2442525/how-to-select-min-and-max-values-of-a-column-in-a-datatable
                List<int> levels = ((DataTable)ViewState["cachedTable"]).
                                    Select("GENERATION = " + (iNextGeneration > 0 ? iNextGeneration - 1 : 0) +
                                    " AND hierarchy LIKE '" + sHierarchy + "%'"). // no space
                                    AsEnumerable().
                                    Select(al => Convert.ToInt32(al.Field<object>("rowNo"))).Distinct().ToList();

                // duplicate hierarchy, display at smallest rowNo will do
                if (levels.Count > 0 && levels.Min() == iCurrRowNo)
                {
                    DataRow[] drResults = ((DataTable)ViewState["cachedTable"]).
                                        Select("GENERATION = " + iNextGeneration + " AND hierarchy LIKE '" + sHierarchy + " %'");

                    foreach (DataRow dr in drResults)
                    {
                        object[] obRow = dr.ItemArray;
                        dt.Rows.Add(obRow);
                    }
                    lv.DataSource = dt;
                    lv.DataBind();
                }
            }
        }

        protected void lst1stLevel_ItemDataBound(object sender, ListViewItemEventArgs e)
        {
            HtmlTableRow row = (HtmlTableRow)e.Item.FindControl("row");
            ListViewDataItem item = (ListViewDataItem)e.Item;
            System.Data.DataRowView drv = (System.Data.DataRowView)item.DataItem;

            int iCurrLvl = 1;
            //populate dynamic cells
            dynamicPopulateRow(row, drv, iCurrLvl);

            //populate nested LV
            var lst2ndLevel = (ListView)e.Item.FindControl("lst2ndLevel");
            populateLV(lst2ndLevel, iCurrLvl + 1, (string)drv["hierarchy"], Convert.ToInt32(drv["rowNo"]));
        }

        protected void lst2ndLevel_ItemDataBound(object sender, ListViewItemEventArgs e)
        {
            HtmlTableRow row = (HtmlTableRow)e.Item.FindControl("row");
            ListViewDataItem item = (ListViewDataItem)e.Item;
            System.Data.DataRowView drv = (System.Data.DataRowView)item.DataItem;
            int iCurrLvl = 2;
            //populate dynamic cells
            dynamicPopulateRow(row, drv, iCurrLvl);

            //populate nested LV
            var lst3rdLevel = (ListView)e.Item.FindControl("lst3rdLevel");
            populateLV(lst3rdLevel, iCurrLvl + 1, (string)drv["hierarchy"], Convert.ToInt32(drv["rowNo"]));
        }

        protected void lst3rdLevel_ItemDataBound(object sender, ListViewItemEventArgs e)
        {
            HtmlTableRow row = (HtmlTableRow)e.Item.FindControl("row");
            ListViewDataItem item = (ListViewDataItem)e.Item;
            System.Data.DataRowView drv = (System.Data.DataRowView)item.DataItem;
            int iCurrLvl = 3;
            //populate dynamic cells
            dynamicPopulateRow(row, drv, iCurrLvl);

            //populate nested LV
            var lst4thLevel = (ListView)e.Item.FindControl("lst4thLevel");
            populateLV(lst4thLevel, iCurrLvl + 1, (string)drv["hierarchy"], Convert.ToInt32(drv["rowNo"]));
        }

        protected void lst4thLevel_ItemDataBound(object sender, ListViewItemEventArgs e)
        {
            HtmlTableRow row = (HtmlTableRow)e.Item.FindControl("row");
            ListViewDataItem item = (ListViewDataItem)e.Item;
            System.Data.DataRowView drv = (System.Data.DataRowView)item.DataItem;
            int iCurrLvl = 4;
            //populate dynamic cells
            dynamicPopulateRow(row, drv, iCurrLvl);
        }
        #endregion
    }
}